from AccessControl import ClassSecurityInfo
from Acquisition import aq_parent
from borg.localrole.interfaces import IFactoryTempFolder
from ftw.contentpage import _
from ftw.contentpage import config
from ftw.contentpage.content.schema import finalize
from ftw.contentpage.interfaces import IAddressBlock
from ftw.geo.interfaces import IGeocodableLocation
from plone.registry.interfaces import IRegistry
from Products.ATContentTypes.content.base import ATCTContent
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.lib.historyaware import HistoryAwareMixin
from simplelayout.base.interfaces import ISimpleLayoutBlock
from zope.component import getUtility
from zope.i18n import translate
from zope.interface import implements

from Products.ATContentTypes.config import HAS_LINGUA_PLONE
if HAS_LINGUA_PLONE:
    from Products.LinguaPlone import public as atapi
else:
    from Products.Archetypes import atapi


schema = atapi.Schema((
    atapi.BooleanField(
        'showTitle',
        schemata='default',
        default=True,
        widget=atapi.BooleanWidget(
        label=_(u'label_show_title',
                default=u'Show Title'))),

    atapi.StringField(
        name='addressTitle',
        schemata='default',
        default_method='getDefaultAddressTitle',
        widget=atapi.StringWidget(
            label=_(u'label_addressTitle',
                    default=u'Address Title'))),

    atapi.StringField(
        name='department',
        schemata='default',
        widget=atapi.StringWidget(
            label=_(u'label_department',
                    default=u'Department'))),

    atapi.StringField(
        name='address',
        schemata='default',
        widget=atapi.StringWidget(
            label=_(u'label_address',
                    default=u'Address'))),

    atapi.StringField(
        name='extraAddressLine',
        schemata='default',
        widget=atapi.StringWidget(
            label=_(u'label_extraAddressLine',
                    default=u'Extra address line'))),

    atapi.StringField(
        name='zip',
        schemata='default',
        widget=atapi.StringWidget(
            label=_(u'label_zip',
                    default=u'ZIP'))),

    atapi.StringField(
        name='city',
        schemata='default',
        widget=atapi.StringWidget(
            label=_(u'label_city',
                    default=u'City'))),

    atapi.StringField(
        name='country',
        schemata='default',
        default_method='getDefaultCountry',
        widget=atapi.StringWidget(
            label=_(u'label_country',
                    default=u'Country'))),

    atapi.StringField(
        name='phone',
        schemata='default',
        widget=atapi.StringWidget(
            label=_(u'label_phone',
                    default=u'Phone'),
            description=_(u'help_phone',
                          default=u''))),

    atapi.StringField(
        name='fax',
        schemata='default',
        widget=atapi.StringWidget(
            label=_(u'label_fax',
                    default=u'Fax'),
            description=_(u'help_fax',
                          default=u''))),

    atapi.StringField(
        name='email',
        schemata='default',
        widget=atapi.StringWidget(
            label=_(u'label_email',
                    default=u'Email'))),

    atapi.StringField(
        'www',
        schemata='default',
        validators=('isURL',),
        widget=atapi.StringWidget(
            label=_(u'label_www', default='WWW'),
            description=_(
                u'help_www',
                default='Please enter a website URL (include http://, https://, etc)'))),

    atapi.BooleanField(
        name='showOpeningHours',
        schemata='default',
        widget=atapi.BooleanWidget(
            label=_(u'label_showOpeningHours',
                    default=u'Show opening hours'),
            description=_(u'help_showOpeningHours',
                          default=u''))),

    atapi.TextField(
        name='openingHours',
        schemata='default',
        default_input_type='text/html',
        default_output_type='text/x-html-safe',
        allowable_content_types=('text/html', ),
        default_content_type='text/html',
        validators=('isTidyHtmlWithCleanup', ),
        widget=atapi.RichWidget(
            label=_(u'label_openingHours',
                    default=u'Opening Hours'),
            description=_(u'help_openingHours',
                          default=u''),
            allow_file_upload=False,
            rows=10)),

    atapi.TextField(
        name='directions',
        schemata='default',
        allowable_content_types=('text/html', ),
        default_content_type='text/html',
        validators=('isTidyHtmlWithCleanup', ),
        default_input_type='text/html',
        default_output_type='text/x-html-safe',
        widget=atapi.RichWidget(
            label=_(u'label_directions',
                    default=u'Directions'),
            description=_(u'help_directions',
                          default=u''),
            allow_file_upload=False,
            rows=10)),
))


addressblock_schema = ATContentTypeSchema.copy() + schema.copy()

addressblock_schema['title'].required = False
addressblock_schema['title'].default_method = 'getDefaultTitle'

allowed_buttons = ('bold', 'link', 'unlink', 'anchor')

addressblock_schema['directions'].widget.allow_buttons = allowed_buttons
addressblock_schema['openingHours'].widget.allow_buttons = allowed_buttons

# Finalize schema
finalize(addressblock_schema, hide=['description'])


class AddressBlock(ATCTContent, HistoryAwareMixin):

    security = ClassSecurityInfo()
    implements(IAddressBlock, ISimpleLayoutBlock, IGeocodableLocation)
    schema = addressblock_schema

    security.declarePrivate('getDefaultCountry')
    def getDefaultCountry(self):
        """ Returns the default country defined in registry.
        """
        registry = getUtility(IRegistry)
        return translate(
            registry.get('ftw.contentpage.addressblock.defaultcountry',
                            ''),
            domain='ftw.contentpage',
            context=self.REQUEST)

    security.declarePrivate('getDefaultTitle')
    def getDefaultTitle(self):
        registry = getUtility(IRegistry)

        return translate(
            registry.get('ftw.contentpage.addressblock.defaulttitle', ''),
            domain='ftw.contentpage',
            context=self.REQUEST)

    security.declarePrivate('getDefaultAddressTitle')
    def getDefaultAddressTitle(self):
        parent = aq_parent(self)
        if IFactoryTempFolder.providedBy(parent):
            parent = aq_parent(aq_parent(parent))
        return parent.Title()

atapi.registerType(AddressBlock, config.PROJECTNAME)



from ftw.colorbox.interfaces import IColorboxSettings
from plone.app.imaging.utils import getAllowedSizes
from plone.registry.interfaces import IRegistry
from Products.Five.browser import BrowserView
from zope.component import getUtility


class ListingBlockGalleryView(BrowserView):
    """Block representation of ListingBlock"""

    @property
    def _build_query(self):
        query = {}
        query['portal_type'] = ['Image']
        return query

    def get_images(self):
        return self.context.listFolderContents(
            contentFilter=self._build_query)

    def _get_box_boundaries(self):
        sizes = getAllowedSizes()
        # Fallback is a plone default scale -> mini
        return sizes.get('listingblock_gallery', 'mini')

    def get_image_large_url(self, img):
        img_url = img.absolute_url()

        registry = getUtility(IRegistry)
        colorbox_settings = registry.forInterface(IColorboxSettings)

        if colorbox_settings.image_size:
            scales = img.restrictedTraverse('@@images')
            scaled = scales.scale('image', scale=colorbox_settings.image_size)

            if scaled:
                return scaled.url

        return img_url



from DateTime import DateTime
from ftw.contentpage.config import HAS_CONTENT_LISTING_BEHAVIOR
from ftw.contentpage.interfaces import ICategorizable, IEventPage
from ftw.contentpage.interfaces import IContentPage
from plone.dexterity.interfaces import IDexterityContent
from plone.indexer.decorator import indexer
from Products.Archetypes.interfaces import IBaseObject
from simplelayout.base.interfaces import ISimpleLayoutBlock


if HAS_CONTENT_LISTING_BEHAVIOR:
    from ftw.contentpage.behaviors.content_categories import IContentCategories


@indexer(ICategorizable)
def categories(obj, **kw):
    if IBaseObject.providedBy(obj):
        return obj.Schema()['content_categories'].get(obj)
    elif IDexterityContent.providedBy(obj) and HAS_CONTENT_LISTING_BEHAVIOR:
        return [item.encode('utf-8') for item in
                IContentCategories(obj).content_categories]
    else:
        return ()


SNIPPETTEXT_FIELDS = {
    'TextBlock': [
        'text',
        'imageCaption',
    ],
    'AddressBlock': [
        'department',
        'address',
        'extraAddressLine',
        'city',
        'country',
        'phone',
        'fax',
        'email',
        'www',
    ],
}


@indexer(IContentPage)
def snippet_text(obj):
    """Text for snippets (aka highlighting) in search results."""
    text = obj.SearchableText()

    # Remove id and title
    for fieldname in ['id', 'title']:
        field = obj.getField(fieldname)
        accessor = field.getIndexAccessor(obj)
        value = accessor()
        text = text.replace(value, '', 1)

    # Include snippet text of the various block types
    block_texts = []
    for block in obj.objectValues():

        if not ISimpleLayoutBlock.providedBy(block):
            continue

        # Only add title if it's visible
        if 'showTitle' in block.Schema():
            if block.getShowTitle():
                block_texts.append(block.Title())
        else:
            block_texts.append(block.Title())

        # Add block-specific fields
        for fieldname in SNIPPETTEXT_FIELDS.get(block.portal_type, []):
            field = block.getField(fieldname)
            accessor = field.getIndexAccessor(block)
            block_texts.append(accessor())

    text += ' '.join(block_texts)

    return text


@indexer(IEventPage)
def start(obj):
    start = obj.start()
    if obj.getWholeDay():
        return DateTime(start.year(), start.month(), start.day(), 0, 0)
    return start


@indexer(IEventPage)
def end(obj):
    end = obj.end()
    if obj.getWholeDay():
        return DateTime(end.year(), end.month(), end.day(), 23, 59)
    return end



from ftw.contentpage.browser.eventlisting import format_date
from plone.app.layout.viewlets import ViewletBase
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class EventDataViewlet(ViewletBase):

    index = ViewPageTemplateFile("event_data.pt")

    def get_date(self):
        return format_date(self.context.start(), self.context.end(),
                           self.context.getWholeDay())
    
    def get_location(self):
        return self.context.getField('location').get(self.context)
        


from zope.publisher.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class TabbedBlockView(BrowserView):
    template = ViewPageTemplateFile('tabs.pt')

    def __call__(self):
        return self.template()

    def children(self):
        return self.context.getFolderContents(
            contentFilter={'object_provides': 'simplelayout.base.interfaces.IOneColumn'},
            full_objects=True)



from ftw.contentpage.interfaces import IAddressBlock
from ftw.upgrade import UpgradeStep
from Products.CMFCore.utils import getToolByName


class ConvertAddressblockToHTML(UpgradeStep):

    def __call__(self):
        portal_transforms = getToolByName(self, 'portal_transforms')

        query = {'object_provides': IAddressBlock.__identifier__}
        for obj in self.catalog_unrestricted_search(query, full_objects=True):
            if obj.directions.getContentType() == 'text/plain':
                directions = portal_transforms.convertTo(
                    'text/html',
                    obj.getRawDirections(),
                    mimetype='text/plain').getData()
                directions = directions.replace('\r', '')
                obj.setDirections(directions)

            if obj.openingHours.getContentType() == 'text/plain':
                openingHours = portal_transforms.convertTo(
                    'text/html',
                    obj.getRawOpeningHours(),
                    mimetype='text/plain').getData()
                openingHours = openingHours.replace('\r', '')
                obj.setOpeningHours(openingHours)



from ftw.builder import Builder
from ftw.builder import create
from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from ftw.testing import MockTestCase
from mocker import ARGS, KWARGS
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.registry.interfaces import IRegistry
from plone.testing.z2 import Browser
from unittest2 import TestCase
from zope.component import getUtility
import transaction


FORM_DATA = {'sender': 'Zaph\xc3\xb6d Beeblebrox',
             'email': 'z.beeblebrox@endofworld.com',
             'subject': 'Don\'t p\xc3\xa4nic',
             'message': '42'}


class TestFeedbackForm(MockTestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def setUp(self):
        super(TestFeedbackForm, self).setUp()
        self.portal = self.layer['portal']
        self.portal_url = self.portal.portal_url()

        self.contentpage = self.portal.get(
            self.portal.invokeFactory('ContentPage', 'contentpage'))
        # Fire all necessary events
        self.contentpage.processForm()
        self.addressblock = self.contentpage.get(
            self.contentpage.invokeFactory('AddressBlock', 'addressblock'))
        self.addressblock.processForm()

        # Set up plone properties
        self.portal.manage_changeProperties(
            {'email_from_name': 'Plone Admin',
             'email_from_address': 'plone@admin.ch'})

        transaction.commit()

        # Browser setup
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False

        self.form = "%s/feedback_view" % self.addressblock.absolute_url()

        # Mock mailhost
        self.mailhost = self.stub()
        self.mock_tool(self.mailhost, 'MailHost')
        self.mails = []
        self.expect(self.mailhost.send(ARGS, KWARGS)).call(
            lambda *args, **kwargs: self.mails.append((args, kwargs)))

        self.replay()

    def _auth(self):
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_NAME, TEST_USER_PASSWORD, ))

    def test_cancel_form(self):
        self._auth()
        self.browser.open(self.form)
        self.browser.getControl('Cancel').click()
        self.assertEqual(self.browser.url,
                         self.contentpage.absolute_url())

    def test_required_fields(self):
        self._auth()
        self.browser.open(self.form)
        self.browser.getControl("Send Mail").click()
        self.assertIn('kssattr-fieldname-form.widgets.sender error',
                      self.browser.contents)
        self.assertIn('kssattr-fieldname-form.widgets.email error',
                      self.browser.contents)
        self.assertIn('kssattr-fieldname-form.widgets.subject error',
                      self.browser.contents)
        self.assertIn('kssattr-fieldname-form.widgets.message error',
                      self.browser.contents)

    def test_send_form(self):
        self._auth()
        self.browser.open(self.form)
        self.browser.getControl(
            name="form.widgets.sender").value = FORM_DATA['sender']
        self.browser.getControl(
            name="form.widgets.email").value = FORM_DATA['email']
        self.browser.getControl(
            name="form.widgets.subject").value = FORM_DATA['subject']
        self.browser.getControl(
            name="form.widgets.message").value = FORM_DATA['message']

        self.browser.getControl('Send Mail').click()

        self.assertEqual(self.browser.url,
                         self.contentpage.absolute_url())

        self.assertIn('The email was sent.', self.browser.contents)
        self.assertEquals(len(self.mails), 1)

        args, kwargs = self.mails.pop()
        self.assertIn('=?utf-8?q?Don=27t_p=C3=A4nic?=', args[0].__str__())
        self.assertIn(FORM_DATA['message'], args[0].__str__())
        self.assertIn(
            'Reply-To: =?utf-8?q?Zaph=C3=B6d_Beeblebrox?='
            ' <z.beeblebrox@endofworld.com>',
            args[0].__str__())
        self.assertIn('From: =?utf-8?q?Plone_Admin?= <plone@admin.ch>',
                      args[0].__str__())

    def test_encode_replyto_always(self):
        """Test that tests if Reply-To header is always encoded."""
        self._auth()
        self.browser.open(self.form)
        self.browser.getControl(
            name="form.widgets.sender").value = 'Hans: Peter'
        self.browser.getControl(
            name="form.widgets.email").value = FORM_DATA['email']
        self.browser.getControl(
            name="form.widgets.subject").value = FORM_DATA['subject']
        self.browser.getControl(
            name="form.widgets.message").value = FORM_DATA['message']

        self.browser.getControl('Send Mail').click()

        self.assertEqual(self.browser.url,
                         self.contentpage.absolute_url())

        args, kwargs = self.mails.pop()
        self.assertIn(
            'Reply-To: =?utf-8?q?Hans=3A_Peter?='
            ' <z.beeblebrox@endofworld.com>',
            args[0].__str__())

    def test_comma_in_sender_name_will_be_replaced(self):
        self._auth()
        self.browser.open(self.form)
        self.browser.getControl(
            name="form.widgets.sender").value = 'Zaph\xc3\xb6d,Beeblebrox'
        self.browser.getControl(
            name="form.widgets.email").value = FORM_DATA['email']
        self.browser.getControl(
            name="form.widgets.subject").value = FORM_DATA['subject']
        self.browser.getControl(
            name="form.widgets.message").value = FORM_DATA['message']

        self.browser.getControl('Send Mail').click()

        args, kwargs = self.mails.pop()
        self.assertIn(
            'Reply-To: =?utf-8?q?Zaph=C3=B6d_Beeblebrox?='
            ' <z.beeblebrox@endofworld.com>',
            args[0].__str__())

    def tearDown(self):
        super(TestFeedbackForm, self).tearDown()
        portal = self.layer['portal']
        portal.manage_delObjects(['contentpage'])
        transaction.commit()


class TestFeedbackFormBrowser(TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager', ])
        login(self.portal, TEST_USER_NAME)

        self.cpage = create(Builder('content page'))
        self.ablock = create(Builder('address block')
                             .within(self.cpage))

    @browsing
    def test_captcha_is_hidden_if_not_configured(self, browser):
        browser.logout().visit(self.ablock, view="feedback_view")

        # ReCaptcha is in the label which is only
        # rendered if the field is visible
        self.assertFalse(browser.css('#formfield-form-widgets-captcha label'),
                         'The captcha should be hidden.')

    @browsing
    def test_captcha_is_used_if_configured(self, browser):
        registry = getUtility(IRegistry)
        registry['plone.formwidget.recaptcha.interfaces.'
                 'IReCaptchaSettings.private_key'] = u'PRIVATE_KEY'
        registry['plone.formwidget.recaptcha.interfaces.'
                 'IReCaptchaSettings.public_key'] = u'PUBLIC_KEY'
        transaction.commit()

        browser.logout().visit(self.ablock, view="feedback_view")

        self.assertTrue(browser.css('#formfield-form-widgets-captcha label'),
                        'The captcha should be visible.')

    @browsing
    def test_captcha_no_component_lookup_error(self, browser):
        """
        A `ComponentLookupError` was thrown if you first opened the feedback
        form in your browser as an unauthorized user and then opened the
        same feedback form as an authorized user (where the captcha field
        should not be displayed).
        This test makes sure that this does not happen any more. Please see
        `ftw.contentpage.browser.feedback.FeedbackForm#updateWidgets`.
        """
        registry = getUtility(IRegistry)
        registry['plone.formwidget.recaptcha.interfaces.'
                 'IReCaptchaSettings.private_key'] = u'PRIVATE_KEY'
        registry['plone.formwidget.recaptcha.interfaces.'
                 'IReCaptchaSettings.public_key'] = u'PUBLIC_KEY'
        transaction.commit()

        browser.logout().visit(self.ablock, view="feedback_view")
        browser.login().visit(self.ablock, view="feedback_view")

        # Our test passes if the page is loading and contains the feedback
        # form.
        self.assertEqual(len(browser.css('#form')), 1)



from ftw.testing import browser
from ftw.testing.pages import Plone

INDEX_XPATH = '//*[contains(concat(" ", normalize-space(@class), " "), ' + \
    '" letter-index ")]'


class SubjectListingView(Plone):

    def visit(self):
        self.visit_portal('@@subject-listing')
        self.assert_body_class('template-subject-listing')
        return self

    @property
    def current_letter(self):
        return map(self._item_to_text,
                   browser().find_by_css('.letter-index .current'))[0]

    @property
    def linked_letters(self):
        return map(self._item_to_text,
                   browser().find_by_css('.letter-index a'))

    @property
    def mimetype_images(self):
        return browser().find_by_css('.subject-mimetype-icon')

    def click_letter(self, letter):
        xpath = INDEX_XPATH + '//a[normalize-space(text())="%s"]' % letter
        elements = browser().find_by_xpath(xpath)
        if len(elements) == 0:
            raise Exception('Letter-link for "%s" not found' % letter)
        elements.click()

    @property
    def subjects(self):
        subjects = browser().find_by_css('.subject')
        return map(self._item_to_text, subjects)

    def get_links_for(self, subject, text_only=False):
        links = browser().find_by_xpath(
            '//td[normalize-space(text())="%s"]/..//a' % subject)

        if text_only:
            return map(self._item_to_text, links)
        else:
            return links

    def _item_to_text(self, item):
        return self.normalize_whitespace(item.text.strip())

    def visit_with_subject_filter(self, value):
        self.visit_portal('@@subject-listing?subject_filter={0}'.format(value))
        self.assert_body_class('template-subject-listing')
        return self


class AuthoritiesView(Plone):

    def visit_on(self, page):
        return self.visit(page, 'authorities_view')

    @property
    def link_labels(self):
        return map(self._item_to_text, browser().find_by_css('#authorities a'))

    @property
    def link_labels_per_column(self):
        columns = []
        for column in browser().find_by_css('#authorities .listing-column'):
            columns.append(map(self._item_to_text,
                               column.find_by_css('a')))
        return columns

    def link_url(self, label):
        links = browser().find_by_xpath(
            '//*[@id="authorities"]//a[normalize-space(text())="%s"]' % label.strip())

        assert len(links) > 0, 'Link with text "%s" not found' % label.strip()
        assert len(links) == 1, 'More than one link with text "%s" found' % (
            label.strip())

        return links[0]['href']

    def _item_to_text(self, item):
        return self.normalize_whitespace(item.text.strip())



from ftw.builder.session import BuilderSession
from ftw.builder.testing import BUILDER_LAYER
from ftw.builder.testing import set_builder_session_factory
from ftw.testing import FunctionalSplinterTesting
from ftw.testing.layer import ComponentRegistryLayer
from plone.app.testing import applyProfile
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import setRoles, TEST_USER_ID, TEST_USER_NAME, login
from plone.testing import z2
from zope.configuration import xmlconfig
from zope.event import notify
from zope.traversing.interfaces import BeforeTraverseEvent
import ftw.contentpage.tests.builders


class ZCMLLayer(ComponentRegistryLayer):
    """Test layer loading the complete package ZCML.
    """

    def setUp(self):
        super(ZCMLLayer, self).setUp()
        import ftw.contentpage.tests
        self.load_zcml_file('tests.zcml', ftw.contentpage.tests)

        import ftw.contentpage.browser
        self.load_zcml_file('configure.zcml', ftw.contentpage.browser)


ZCML_LAYER = ZCMLLayer()


def functional_session_factory():
    sess = BuilderSession()
    sess.auto_commit = True
    return sess


class FtwContentPageLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE, BUILDER_LAYER)

    def setUpZope(self, app, configurationContext):
        # Load ZCML
        import ftw.contentpage
        import simplelayout.base
        import ftw.geo
        import collective.geo.settings
        import collective.geo.openlayers
        import collective.geo.geographer
        import collective.geo.contentlocations
        import collective.geo.kml
        import plone.formwidget.contenttree

        xmlconfig.file('configure.zcml', ftw.contentpage,
                       context=configurationContext)
        xmlconfig.file('configure.zcml', simplelayout.base,
                       context=configurationContext)
        xmlconfig.file('configure.zcml', ftw.geo,
                       context=configurationContext)
        xmlconfig.file('configure.zcml', collective.geo.settings,
                       context=configurationContext)
        xmlconfig.file('configure.zcml', collective.geo.openlayers,
                       context=configurationContext)
        xmlconfig.file('configure.zcml', collective.geo.geographer,
                       context=configurationContext)
        xmlconfig.file('configure.zcml', collective.geo.contentlocations,
                       context=configurationContext)
        xmlconfig.file('configure.zcml', collective.geo.kml,
                       context=configurationContext)

        xmlconfig.file('configure.zcml', plone.formwidget.contenttree,
                       context=configurationContext)

        # installProduct() is *only* necessary for packages outside
        # the Products.* namespace which are also declared as Zope 2
        # products, using <five:registerPackage /> in ZCML.
        z2.installProduct(app, 'ftw.contentpage')
        z2.installProduct(app, 'simplelayout.base')

    def setUpPloneSite(self, portal):
        # Install into Plone site using portal_setup
        applyProfile(portal, 'ftw.contentpage:default')
        applyProfile(portal, 'simplelayout.base:default')
        applyProfile(portal, 'plone.formwidget.recaptcha:default')
        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)


class FunctionalBrowserlayerTesting(FunctionalSplinterTesting):
    """Support browserlayer"""

    def setUpEnvironment(self, portal):
        super(FunctionalBrowserlayerTesting, self).setUpEnvironment(portal)

        notify(BeforeTraverseEvent(portal, portal.REQUEST))


class IntegrationBrowserlayerTesting(IntegrationTesting):
    """Support browserlayer"""

    def setUpEnvironment(self, portal):
        super(IntegrationBrowserlayerTesting, self).setUpEnvironment(portal)

        notify(BeforeTraverseEvent(portal, portal.REQUEST))


FTW_CONTENTPAGE_FIXTURE = FtwContentPageLayer()
FTW_CONTENTPAGE_INTEGRATION_TESTING = IntegrationBrowserlayerTesting(
    bases=(FTW_CONTENTPAGE_FIXTURE, ), name="FtwContentPage:Integration")
FTW_CONTENTPAGE_FUNCTIONAL_TESTING = FunctionalBrowserlayerTesting(
    bases=(FTW_CONTENTPAGE_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name="FtwContentPage:Functional")



from DateTime import DateTime
from ftw.builder import Builder
from ftw.builder import create
from ftw.contentpage.tests import FunctionalTestCase
from ftw.testbrowser import browsing


class TestNewsArchivePortlet(FunctionalTestCase):

    def setUp(self):
        super(TestNewsArchivePortlet, self).setUp()
        self.grant('Manager')

    @browsing
    def test_news_archive_portlet(self, browser):
        news_folder = create(Builder('news folder'))

        create(Builder('news')
               .within(news_folder)
               .having(effectiveDate=DateTime(2013, 3, 20, 10, 0)))

        create(Builder('news')
               .within(news_folder)
               .having(effectiveDate=DateTime(2013, 5, 26, 10, 0)))

        create(Builder('news')
               .within(news_folder)
               .having(effectiveDate=DateTime(2013, 6, 28, 10, 0)))

        create(Builder('news archive portlet').within(news_folder))

        browser.login().visit(news_folder)

        self.assertTrue(
            browser.css('.portlet.portletArchiveListing'),
            msg='There should be an news archive portlet, but it isn\'t.'
        )

        self.assertEqual(
            3,
            len(browser.css('.portlet.portletArchiveListing .portletItem li.month')),
            msg='There should be 3 entries in the news archive portlet.'
        )

    @browsing
    def test_news_archive_portlet_renders_default_title(self, browser):
        news_folder = create(Builder('news folder'))

        create(Builder('news')
               .within(news_folder)
               .having(effectiveDate=DateTime(2014, 2, 2)))

        create(Builder('news archive portlet').within(news_folder))

        browser.login().visit(news_folder)
        self.assertEqual(
            ['Archive'],
            browser.css('dl.portletArchiveListing dt').text
        )

    @browsing
    def test_news_archive_portlet_renders_custom_title(self, browser):
        news_folder = create(Builder('news folder'))

        create(Builder('news')
               .within(news_folder)
               .having(effectiveDate=DateTime(2014, 2, 2)))

        create(Builder('news archive portlet').within(news_folder))
        browser.login()

        # Make sure the portlet renders the default title.
        browser.visit(news_folder)
        self.assertEqual(
            ['Archive'],
            browser.css('dl.portletArchiveListing dt').text
        )

        # Now let's change the title of the portlet.
        browser.visit(news_folder, view='@@manage-portlets')
        browser.find('News Archive Portlet').click()
        browser.fill({'Title': u'Our Custom Title'}).submit()

        # Make sure the portlet renders the custom title.
        browser.visit(news_folder)
        self.assertEqual(
            ['Our Custom Title'],
            browser.css('dl.portletArchiveListing dt').text
        )



from ftw.builder import Builder
from ftw.builder import create
from ftw.contentpage.browser.subject_listing import make_sortable
from ftw.contentpage.browser.subject_listing import normalized_first_letter
from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING
from ftw.contentpage.tests.pages import SubjectListingView
from ftw.testing import browser
from plone.registry.interfaces import IRegistry
from unittest2 import TestCase
from zope.component import getUtility
import transaction


class TestNormalizedFirstLetter(TestCase):

    def test_returns_uppercase_letter(self):
        self.assertEquals('T', normalized_first_letter('test'))

    def test_normalizes_umlauts(self):
        self.assertEquals('A', normalized_first_letter('\xc3\xa4bc'))


class TestMakeSortable(TestCase):

    def test_text_is_lowercased(self):
        self.assertEquals('foo',
                          make_sortable('FoO'))

    def test_umlauts_are_normalized(self):
        self.assertEquals('loffel',
                          make_sortable('L\xc3\xb6ffel'))

    def test_cedillas_are_removed(self):
        self.assertEquals('francais',
                          make_sortable('fran\xc3\xa7ais'))


class TestAlphabeticalSubjectListing(TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def test_subjects_are_ordered_alphabetically(self):
        create(Builder('content page')
               .having(subject=['Airport']))

        create(Builder('content page')
               .having(subject=['Address modification',
                                'AZ']))

        view = SubjectListingView().visit()
        self.assertEquals(['Address modification', 'Airport', 'AZ'],
                          view.subjects)

    def test_subjects_with_umlauts_are_ordered_correctly(self):
        create(Builder('content page')
               .having(subject=['Grippe',
                                'Gr\xc3\xa4ber']))

        view = SubjectListingView().visit()
        self.assertEquals(['Gr\xc3\xa4ber'.decode('utf-8'),
                           'Grippe'.decode('utf-8')],
                          view.subjects)

    def test_content_links_are_ordered_alphabetically(self):
        create(Builder('content page')
               .titled('Foo')
               .having(subject=['Airport']))
        create(Builder('content page')
               .titled('Bar')
               .having(subject=['Airport']))
        create(Builder('content page')
               .titled('BAZ')
               .having(subject=['Airport']))

        view = SubjectListingView().visit()
        self.assertEquals(['Bar', 'BAZ', 'Foo'],
                          view.get_links_for('Airport', text_only=True))

    def test_content_links_with_umlauts_are_ordered_correctly(self):
        create(Builder('content page')
               .titled('G\xc3\xa4rtnerei')
               .having(subject=['Garten']))

        create(Builder('content page')
               .titled('Gr\xc3\xbcnfl\xc3\xa4chen')
               .having(subject=['Garten']))

        view = SubjectListingView().visit()
        self.assertEquals(['G\xc3\xa4rtnerei'.decode('utf-8'),
                           'Gr\xc3\xbcnfl\xc3\xa4chen'.decode('utf-8')],
                          view.get_links_for('Garten', text_only=True))

    def test_content_is_properly_linked(self):
        page = create(Builder('content page')
                      .having(subject=['Airport']))

        view = SubjectListingView().visit()
        view.get_links_for('Airport').first.click()
        self.assertEquals(page.absolute_url(), browser().url)

    def test_first_letter_with_content_is_selected(self):
        create(Builder('content page')
               .having(subject=['building application', 'crafting', '3D']))

        view = SubjectListingView().visit()
        self.assertEquals('B', view.current_letter)

    def test_only_letters_with_content_are_linked(self):
        create(Builder('content page')
               .having(subject=['building application',
                                'crafting',
                                'manufactoring']))

        view = SubjectListingView().visit()
        self.assertEquals(['B', 'C', 'M'], view.linked_letters)

    def test_navigating_in_letter_index(self):
        create(Builder('content page')
               .having(subject=['building application',
                                'crafting',
                                'manufactoring']))
        view = SubjectListingView().visit()

        self.assertEquals('B', view.current_letter)
        view.click_letter('C')
        self.assertEquals('C', view.current_letter)

    def test_only_content_of_current_is_listed(self):
        create(Builder('content page')
               .having(subject=['building application',
                                'budget',
                                'crafting']))

        create(Builder('content page')
               .having(subject=['crafting']))

        view = SubjectListingView().visit()
        self.assertEquals('B', view.current_letter)
        self.assertEquals(['budget', 'building application'],
                          view.subjects)

        view.click_letter('C')
        self.assertEquals(['crafting'],
                          view.subjects)

    def test_umlauts_are_normalized_for_letter_index(self):
        create(Builder('content page')
               .having(subject=['\xc3\xa4bc']))

        view = SubjectListingView().visit()
        self.assertEquals('A', view.current_letter)
        self.assertEquals([u'\xe4bc'], view.subjects)

    def test_non_alphabetic_subjects_are_listed_under_number_sign(self):
        create(Builder('content page')
               .having(subject=['3D',
                                '_X']))

        view = SubjectListingView().visit()
        view.click_letter('#')
        self.assertEquals(['3D', '_X'], view.subjects)

    def test_lists_only_content_pages_by_default(self):
        create(Builder('content page')
               .titled('A page')
               .having(subject=['budget']))

        create(Builder('news')
               .titled('A news')
               .having(subject=['budget']))

        view = SubjectListingView().visit()
        self.assertEquals(['A page'],
                          view.get_links_for('budget', text_only=True))

    def test_listed_types_are_configurable_in_registry(self):
        registry = getUtility(IRegistry)
        registry['ftw.contentpage.subjectlisting.portal_types'] = [
            u'ContentPage',
            u'News']

        create(Builder('content page')
               .titled('A page')
               .having(subject=['budget']))

        create(Builder('news')
               .titled('A news')
               .having(subject=['budget']))

        view = SubjectListingView().visit()
        self.assertEquals(['A news', 'A page'],
                          view.get_links_for('budget', text_only=True))

    def test_visiting_page_with_no_contents_does_not_crash(self):
        SubjectListingView().visit()

        self.assertEquals(
            'There is no content to list.',
            browser().find_by_css('#content .no-contents').text.strip())

    def test_dont_show_mimetype_icon_per_default(self):
        create(Builder('content page')
               .titled('A page')
               .having(subject=['budget']))

        view = SubjectListingView().visit()
        self.assertEquals([], view.mimetype_images)

    def test_show_mimetype_icon_if_set_in_registry(self):
        registry = getUtility(IRegistry)
        registry['ftw.contentpage.subjectlisting.show_mimetype_icon'] = True

        create(Builder('content page')
               .titled('A page')
               .having(subject=['budget']))

        view = SubjectListingView().visit()
        self.assertEquals(1, len(view.mimetype_images))

    def test_filter_subjects_if_context_property_is_set(self):
        create(Builder('content page')
               .titled('A page')
               .having(subject=['A budget', 'A finance']))

        create(Builder('content page')
               .titled('A page')
               .having(subject=['A logistic']))

        self.layer['portal']._setProperty(
            'subject_filter', 'A budget', 'string')
        transaction.commit()

        view = SubjectListingView().visit()

        self.assertEquals([u'A finance'], view.subjects)

    def test_filter_subjects_if_request_variable_is_set(self):
        create(Builder('content page')
               .titled('A page')
               .having(subject=['A budget', 'A finance']))

        create(Builder('content page')
               .titled('A page')
               .having(subject=['A logistic']))

        view = SubjectListingView().visit_with_subject_filter('A budget')

        self.assertEquals([u'A finance'], view.subjects)

    def test_request_filter_wins_agains_context_property(self):
        create(Builder('content page')
               .titled('A page')
               .having(subject=['A budget', 'A finance']))

        create(Builder('content page')
               .titled('A page')
               .having(subject=['A factory', 'A logistic']))

        self.layer['portal']._setProperty(
            'subject_filter', 'A factory', 'string')
        transaction.commit()

        view = SubjectListingView().visit_with_subject_filter('A budget')

        self.assertEquals([u'A finance'], view.subjects)

    def test_ignore_the_filtered_subject_in_the_listing(self):
        create(Builder('content page')
               .titled('A page')
               .having(subject=['A budget', 'A finance', 'A money']))

        create(Builder('content page')
               .titled('A page')
               .having(subject=['A earn', 'A budget']))

        view = SubjectListingView().visit_with_subject_filter('A budget')

        self.assertEquals([u'A earn', 'A finance', 'A money'], view.subjects)

    def test_only_list_subjects_related_to_objects_with_filtered_subject(self):
        create(Builder('content page')
               .titled('A page')
               .having(subject=['A budget', 'A finance', 'A money']))

        create(Builder('content page')
               .titled('A page')
               .having(subject=['A finance', 'A earn']))

        view = SubjectListingView().visit_with_subject_filter('A budget')

        self.assertEquals([u'A finance', 'A money'], view.subjects)

    def test_letters_assumes_subject_filter(self):
        create(Builder('content page')
               .titled('A page')
               .having(subject=['budget', 'finance', 'money']))

        create(Builder('content page')
               .titled('A page')
               .having(subject=['finance', 'earn']))

        view = SubjectListingView().visit()
        self.assertEquals(['B', 'E', 'F', 'M'], view.linked_letters)

        view = SubjectListingView().visit_with_subject_filter('budget')
        self.assertEquals(['F', 'M'], view.linked_letters)

        view.click_letter('M')

        self.assertEquals(['F', 'M'], view.linked_letters)

    def test_not_break_if_filtered_subject_have_no_additional_subjects(self):
        create(Builder('content page')
               .titled('A page')
               .having(subject=['budget']))

        SubjectListingView().visit_with_subject_filter('budget')

        self.assertEquals(
            'There is no content to list.',
            browser().find_by_css('#content .no-contents').text.strip())



from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from collections import defaultdict
from plone.memoize import instance
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
import unicodedata


ALPHABET = map(chr, range(ord('A'), ord('Z') + 1))
LETTERS = ALPHABET + ['#']


def normalized_first_letter(text):
    text = make_sortable(text)
    return text[0].upper()


def make_sortable(text):
    text = text.lower()
    text = text.decode('utf-8')
    normalized = unicodedata.normalize('NFKD', text)
    text = u''.join([c for c in normalized if not unicodedata.combining(c)])
    text = text.encode('utf-8')
    return text


class AlphabeticalSubjectListing(BrowserView):

    def __init__(self, *args, **kwargs):
        super(AlphabeticalSubjectListing, self).__init__(*args, **kwargs)
        self.letter = None

    def subjects(self):
        current_letter = self.get_current_letter()

        subjects = []
        for subject in self.available_subjects():

            if current_letter == '#':
                if normalized_first_letter(subject) not in ALPHABET:
                    subjects.append(subject)

            else:
                if normalized_first_letter(subject) == current_letter:
                    subjects.append(subject)

        return subjects

    def result(self):
        content = self.get_content_by_subjects(self.subjects())
        result = []

        for subject, brains in content.items():
            brains = sorted(brains,
                            key=lambda brain:make_sortable(brain.Title))
            result.append({
                    'subject': subject,
                    'brains': brains})

        return sorted(result,
                      key=lambda item: make_sortable(item['subject']))

    def get_content_by_subjects(self, subjects):
        catalog = getToolByName(self.context, 'portal_catalog')
        result = defaultdict(list)
        query = {'Subject': subjects,
                 'portal_type': self.portal_types_to_list}

        for brain in catalog(query):

            # Ignore brains without the filtered subject
            if self.ignore_brain(brain.Subject):
                continue

            for subj in brain.Subject:

                if subj not in subjects:
                    continue

                result[subj].append(brain)
        return result

    def ignore_brain(self, brain_subjects):
        if not self.subject_filter:
            return False

        if self.subject_filter in brain_subjects:
            return False
        return True

    def letters(self):
        letters_with_content = self._letters_with_content
        current_letter = self.get_current_letter()

        for letter in LETTERS:
            if letter == '#':
                character = '!'
            else:
                character = letter
            yield {'label': letter,
                   'has_contents': letter in letters_with_content,
                   'link': self.get_letter_link(character),
                   'current': letter == current_letter}

    def get_letter_link(self, character):
        link = '/'.join((self.context.absolute_url(),
                                     '@@subject-listing',
                                     character))

        if self.subject_filter:
            link = "{0}?subject_filter={1}".format(
                link,
                self.subject_filter)

        return link

    def has_contents(self):
        return bool(len(self._letters_with_content))

    def publishTraverse(self, request, name):
        if name == '!':
            self.letter = '#'
        else:
            self.letter = name
        return self

    @property
    def subject_filter(self):
        return self.request.form.get(
            'subject_filter', self.context.getProperty('subject_filter'))

    def available_subjects(self):
        catalog = getToolByName(self.context, 'portal_catalog')

        if not self.subject_filter:
            return catalog.uniqueValuesFor("Subject")

        brains = catalog.searchResults(Subject=self.subject_filter)
        subjects = []

        for brain in brains:
            for subject in brain.Subject:

                # Exclude the filtered subjects
                if subject == self.subject_filter:
                    continue

                # Subject already exists
                if subject in subjects:
                    continue

                subjects.append(subject)

        return subjects

    def get_mimetype_icon(self, brain):
        registry = getUtility(IRegistry)
        if not registry['ftw.contentpage.subjectlisting.show_mimetype_icon']:
            return None

        icon = brain.getIcon

        if not icon:
            return None

        portal_url = getToolByName(self.context, 'portal_url')

        return '%s/%s' % (portal_url(), icon)

    @instance.memoize
    def get_current_letter(self):
        if self.letter is not None:
            return self.letter

        return self._letters_with_content[0]

    @property
    @instance.memoize
    def _letters_with_content(self):
        values = self.available_subjects()

        letters = set([])
        for subject in values:
            first_letter = normalized_first_letter(subject)
            if first_letter in ALPHABET:
                letters.add(first_letter)
            else:
                letters.add('#')

        return sorted(letters, key=LETTERS.index)

    @property
    def portal_types_to_list(self):
        registry = getUtility(IRegistry)
        return registry['ftw.contentpage.subjectlisting.portal_types']



from ftw.geo.interfaces import IGeocodableLocation
from zope.component import adapts
from zope.interface import implements
from ftw.contentpage.interfaces import IAddressBlock


class AddressBlockLocationAdapter(object):
    """Adapter that is able to represent the location of an MyType in
    a geocodable string form.
    """
    implements(IGeocodableLocation)
    adapts(IAddressBlock)

    def __init__(self, context):
        self.context = context

    def getLocationString(self):
        """Build a geocodable location string from the AddressBlocks address
        related fields.
        """
        street = self.context.getAddress().strip()
        # Remove Postfach form street, otherwise Google geocoder API will
        # return wrong results
        street = street.replace('Postfach', '').replace('\r', '').strip()
        zip_code = self.context.getZip()
        city = self.context.getCity()
        country = self.context.getCountry()

        location = ', '.join([street, zip_code, city, country])

        # We need at least something other than country to be defined,
        # otherwise we shouldn't try to do a geocode lookup
        if not (street or zip_code or city):
            return ''

        return location



from ftw.contentpage import _
from plone.autoform import directives
from plone.autoform.directives import write_permission
from plone.autoform.interfaces import IFormFieldProvider
from plone.behavior.annotation import AnnotationsFactoryImpl
from plone.supermodel import model
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from zope import schema
from zope.interface import alsoProvides


class IContentCategories(model.Schema):

    """Content categories schema"""

    content_categories = schema.Tuple(
        title=_(u'label_categories', default=u'Categories'),
        description=_(u'help_categories',
                      default=u'Category for contentlisting'),
        value_type=schema.Choice(title=_(u"categories"),
                                 vocabulary='ftw.contentpage.contentcategories'),
        required=False,
        missing_value=(),
    )
    directives.widget('content_categories', CheckBoxFieldWidget)

    write_permission(new_content_categories='ftw.contentpage.content_categories_behavior.add_new_categories')
    new_content_categories = schema.Tuple(
        title=_(u'label_new_categories', default=u'New categories'),
        description=_(u'help_new_categories',
                      default=u'New categories for contentlisting'),
        value_type=schema.TextLine(),
        required=False,
        missing_value=(),
    )


alsoProvides(IContentCategories, IFormFieldProvider)


class ContentCategoriesStorage(object):

    def __init__(self, context):
        self.context = context
        self.annoation_storage = AnnotationsFactoryImpl(
            self.context,
            IContentCategories
        )

    @property
    def content_categories(self):
        return self.annoation_storage.content_categories

    @content_categories.setter
    def content_categories(self, value):
        self.annoation_storage.content_categories = value

    @property
    def new_content_categories(self):
        return ()

    @new_content_categories.setter
    def new_content_categories(self, value):
        if value is None:
            return ()
        else:
            categories = tuple(self.annoation_storage.content_categories)
            self.annoation_storage.content_categories = tuple(set(
                categories + value
            ))



from DateTime import DateTime
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.i18nl10n import monthname_msgid
from Products.CMFPlone.utils import base_hasattr
from zope.i18n import translate


def zLocalizedTime(request, time, long_format=False):
    """Convert time to localized time
    """
    month_msgid = monthname_msgid(time.strftime("%m"))
    month = translate(month_msgid, domain='plonelocales',
                      context=request)

    return u"%s" % (month)


class ArchiveSummary(object):

    def __init__(self, context, request, interfaces, datefield, viewname):
        self.context = context
        self.request = request
        self.interfaces = interfaces
        self.datefield = datefield
        self.viewname = viewname
        self.selected_year = None
        self.selected_month = None

    def __call__(self):
        self._set_selected_archive()

        entries = self._get_archive_entries()
        counter = self._count_entries(entries)

        result = []
        year_numbers = sorted(counter, reverse=True)

        for year_number in year_numbers:

            year = counter.get(year_number)
            months = year.get('months')
            month_numbers = sorted(months, reverse=True)
            month_list = []

            for month_number in month_numbers:
                date = '%s/%s/01' % (year_number, month_number)

                month_list.append(dict(
                    title=zLocalizedTime(self.request, DateTime(date)),
                    number=months.get(month_number),
                    url=self._get_archive_url(date),
                    mark=[self.selected_year, self.selected_month] == [
                        year_number, month_number]
                ))

            result.append(dict(
                title=year_number,
                number=year.get('num'),
                months=month_list,
                mark=self.selected_year == year_number
            ))

        return result

    def _set_selected_archive(self):
        selected_archive = self.request.get('archive')
        if selected_archive:
            self.selected_year = selected_archive.split('/')[0]
            self.selected_month = selected_archive.split('/')[1]

    def _get_archive_url(self, date):
        return '%s/%s?archive=%s' % (
            self.context.absolute_url(),
            self.viewname,
            date)

    def _get_archive_entries(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        return catalog(**self._get_query())

    def _get_query(self):
        query = {}
        if base_hasattr(self.context, 'getTranslations'):
            roots = self.context.getTranslations(
                review_state=False).values()
            root_path = ['/'.join(br.getPhysicalPath()) for br in roots]
            query['Language'] = 'all'
        else:
            root_path = '/'.join(self.context.getPhysicalPath())

        query['path'] = root_path
        query['object_provides'] = self.interfaces

        return query

    def _count_entries(self, entries):
        """Return a summary map like:

        {'2009': {
            'num': 6,
            'months': {
                '01': 4,
                '02': 2}
            }
        }
        """
        summary = {}

        for entry in entries:

            date = getattr(entry, self.datefield)
            if not date:
                continue

            if date.year() <= 1900:
                continue

            year_name = date.strftime('%Y')
            month_name = date.strftime('%m')

            year_summary = summary.get(year_name, {})
            month_summary = year_summary.get('months', {})

            # Increase month
            month_summary.update(
                {month_name: month_summary.get(month_name, 0) + 1})
            year_summary.update({'months': month_summary})

            # Increase year
            year_summary.update({'num': year_summary.get('num', 0) + 1})
            summary.update({year_name: year_summary})

        return summary



from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from plone.app.testing import TEST_USER_NAME, TEST_USER_PASSWORD
from plone.testing.z2 import Browser
from simplelayout.base.utils import IBlockControl
from unittest2 import TestCase
from zope.component import getUtility
from zope.component import queryMultiAdapter
import os
import transaction


class TestTextBlockView(TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def setUp(self):
        super(TestTextBlockView, self).setUp()
        self.portal = self.layer['portal']
        self.portal_url = self.portal.portal_url()

        self.contentpage = self.portal.get(
            self.portal.invokeFactory('ContentPage', 'contentpage'))
        # Fire all necessary events
        self.contentpage.processForm()
        self.contentpage.reindexObject()
        transaction.commit()

        # Browser setup
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False

    def _create_textblock(self):
        textblock = self.contentpage.get(
            self.contentpage.invokeFactory('TextBlock', 'textblock'))
        # Fire all necessary events
        textblock.processForm()
        transaction.commit()
        return textblock

    def _auth(self):
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_NAME, TEST_USER_PASSWORD, ))

    def test_addressblock_view(self):
        textblock = self._create_textblock()
        self._auth()
        self.browser.open(self.contentpage.absolute_url())
        self.assertIn(textblock.getId(), self.browser.contents)
        self.assertIn('simplelayout-block-wrapper TextBlock',
                      self.browser.contents)

    def test_show_title(self):
        textblock = self._create_textblock()
        test_title = 'This is the title'
        textblock.setTitle(test_title)
        view = queryMultiAdapter((textblock, textblock.REQUEST),
                                 name='block_view')

        self.assertNotIn(test_title, view.index())

        textblock.setShowTitle(True)
        self.assertIn(test_title, view.index())

    def test_text_not_required(self):
        textblock = self._create_textblock()
        test_text = 'This is the Text'
        view = queryMultiAdapter((textblock, textblock.REQUEST),
                                 name='block_view')

        self.assertNotIn(test_text, view.index())

        textblock.setText(test_text)
        self.assertIn(test_text, view.index())

    def test_has_image(self):
        textblock = self._create_textblock()
        self._auth()
        self.browser.open(self.contentpage.absolute_url())
        self.assertNotIn('sl-img-wrapper', self.browser.contents)

        view = queryMultiAdapter((textblock, textblock.REQUEST),
                                 name='block_view')
        self.assertFalse(view.has_image())

        # Add Image
        dummy = open("%s/dummy.png" % os.path.split(__file__)[0], 'r')
        dummy.seek(0)
        textblock.setImage(dummy)
        textblock.processForm()
        transaction.commit()

        self.assertTrue(view.has_image())

        self.browser.open(self.contentpage.absolute_url())
        self.assertIn('sl-img-wrapper', self.browser.contents)

    def test_get_image_tag(self):
        textblock = self._create_textblock()
        view = queryMultiAdapter((textblock, textblock.REQUEST),
                                 name='block_view')
        # No image uploaded
        self.assertEquals(view.get_image_tag(), '')

        # Add Image
        dummy = open("%s/dummy.png" % os.path.split(__file__)[0], 'r')
        dummy.seek(0)
        textblock.setImage(dummy)
        textblock.setImageAltText('image alt text')
        textblock.processForm()

        self.assertIn('alt="image alt text"', view.get_image_tag())
        textblock.setImageCaption('image caption')
        self.assertIn('title="image caption"', view.get_image_tag())

        # Change view
        converter = getUtility(IBlockControl, name='block-layout')
        converter.update(self.contentpage,
                         textblock,
                         self.contentpage.REQUEST,
                         layout='no-image',
                         viewname='block-view')

        self.assertEquals(view.get_image_tag(), '')

    def test_image_wrapper_style(self):
        textblock = self._create_textblock()
        view = queryMultiAdapter((textblock, textblock.REQUEST),
                                 name='block_view')

        # Add Image
        dummy = open("%s/dummy.png" % os.path.split(__file__)[0], 'r')
        dummy.seek(0)
        textblock.setImage(dummy)
        textblock.processForm()
        transaction.commit()

        width = view.image_wrapper_style()[7:-2]
        self.assertIn(width, view.get_image_tag())
        self.browser.open(self.contentpage.absolute_url())
        self.assertIn(view.image_wrapper_style(),
                      self.browser.contents)

    def test_image_with_umlauts_in_filename(self):
        textblock = self._create_textblock()
        view = queryMultiAdapter((textblock, textblock.REQUEST),
                                  name='block_view')

        with open("%s/dummy.png" % os.path.split(__file__)[0], 'r') as dummy:
            textblock.setImage(dummy)
        textblock.getImage().filename = 'K\xc3\xbcche.png'
        self.assertIn(u'K\xfcche.png', view.get_image_tag())

    def test_get_css_klass(self):
        textblock = self._create_textblock()
        view = queryMultiAdapter((textblock, textblock.REQUEST),
                                 name='block_view')

        # Init
        self.assertEquals(view.get_css_klass(), 'sl-img-small')

        # Change view
        converter = getUtility(IBlockControl, name='block-layout')
        converter.update(self.contentpage,
                         textblock,
                         self.contentpage.REQUEST,
                         layout='no-image',
                         viewname='block-view')

        view = queryMultiAdapter((textblock, textblock.REQUEST),
                                 name='block_view')
        self.assertEquals(view.get_css_klass(), 'sl-img-no-image')

    @browsing
    def test_image_caption_not_linked(self, browser):
        """
        The image caption has accidentally been linked to the image
        overlay. This test makes sure that this is no longer the case.
        """
        textblock = self._create_textblock()

        # Add Image
        dummy = open("%s/dummy.png" % os.path.split(__file__)[0], 'r')
        dummy.seek(0)
        textblock.setImage(dummy)
        textblock.setImageAltText('image alt text')
        # Set the image caption, which our test is based on.
        textblock.setImageCaption('image caption')
        # Enable opening the image in an overlay, this generates the link.
        textblock.setImageClickable(True)
        textblock.processForm()
        transaction.commit()

        browser.login().visit(self.contentpage)

        # The caption must not be inside the link.
        self.assertEqual([], browser.css(".sl-img-wrapper a p"))

        # But instead, the image caption must be outside of the link.
        self.assertEqual(['image caption'],
                         browser.css(".sl-img-wrapper p").text)

    def tearDown(self):
        super(TestTextBlockView, self).tearDown()
        portal = self.layer['portal']
        portal.manage_delObjects(['contentpage'])

        transaction.commit()



from ftw.contentpage.testing import FTW_CONTENTPAGE_INTEGRATION_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from unittest2 import TestCase
from plone.indexer.wrapper import IndexableObjectWrapper


class TestSnippetText(TestCase):
    
    layer = FTW_CONTENTPAGE_INTEGRATION_TESTING

    def setUp(self):
        portal = self.layer['portal']
        setRoles(portal, TEST_USER_ID, ['Manager'])
        self.page = portal[portal.invokeFactory('ContentPage', 'page',
            title=u"The ContentPage Title",
            description=u"The ContentPage Description")]
        self.wrapped = IndexableObjectWrapper(self.page, portal.portal_catalog)

    def test_contentpage_id_not_in_snippettext(self):
        self.assertNotIn('page', self.wrapped.snippetText)

    def test_contentpage_title_not_in_snippettext(self):
        self.assertNotIn("The ContentPage Title", self.wrapped.snippetText)

    def test_contentpage_description_in_snippetttext(self):
        self.assertIn("The ContentPage Description", self.wrapped.snippetText)

    def test_textblock_title_and_id_not_in_snippettext(self):
        self.page.invokeFactory('TextBlock', 'block',
            title=u'The TextBlock Title')
        self.assertNotIn('block', self.wrapped.snippetText)
        self.assertNotIn('The TextBlock Title', self.wrapped.snippetText)

    def test_textblock_title_in_snippettext_if_visible(self):
        self.page.invokeFactory('TextBlock', 'block',
            title=u'The TextBlock Title', showTitle=True)
        self.assertIn('The TextBlock Title', self.wrapped.snippetText)

    def test_textblock_text_in_snippettext(self):
        self.page.invokeFactory('TextBlock', 'block',
            text=u'The TextBlock Text')
        self.assertIn('The TextBlock Text', self.wrapped.snippetText)

    def test_addressblock_fields_in_snippettext(self):
        self.page.invokeFactory('AddressBlock', 'block',
            address=u'1st Avenue', city='Gotham City')
        self.assertIn('1st Avenue', self.wrapped.snippetText)
        self.assertIn('Gotham City', self.wrapped.snippetText)

    def test_contentpage_in_contentpage_is_ignored_in_snippettext(self):
        self.page.invokeFactory('ContentPage', 'subpage',
            description="A ContentPage in a ContentPage")
        self.assertNotIn("A Contentpage in a ContentPage",
                         self.wrapped.snippetText)



from ftw.contentpage.content.schema import DEFAULT_TO_HIDE
from ftw.contentpage.content.schema import finalize
from ftw.contentpage.testing import FTW_CONTENTPAGE_INTEGRATION_TESTING
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.CMFCore.permissions import ManagePortal
from unittest2 import TestCase


class TestAddressBlockCreation(TestCase):

    layer = FTW_CONTENTPAGE_INTEGRATION_TESTING

    def setUp(self):
        super(TestAddressBlockCreation, self).setUp()
        self.portal = self.layer['portal']
        self.portal_url = self.portal.portal_url()

    def test_finalize_schema(self):
        schema = ATContentTypeSchema.copy()
        finalize(schema)

        for name in DEFAULT_TO_HIDE:
            if name in schema:
                self.assertEquals(ManagePortal, schema[name].write_permission)
                self.assertEquals({'view': 'invisible', 'edit': 'invisible'},
                                  schema[name].widget.visible)

        self.assertTrue(schema['excludeFromNav'].default)

    def test_finalize_schema_additional_hide(self):
        schema = ATContentTypeSchema.copy()
        finalize(schema, hide=['title'])

        self.assertEquals(ManagePortal, schema['title'].write_permission)
        self.assertEquals({'view': 'invisible', 'edit': 'invisible'},
                          schema['title'].widget.visible)

    def test_finalize_schema_do_not_hide(self):
        schema = ATContentTypeSchema.copy()
        finalize(schema, show=['subject'])

        self.assertNotEqual(ManagePortal, schema['subject'].write_permission)
        self.assertNotEquals({'view': 'invisible', 'edit': 'invisible'},
                             schema['subject'].widget.visible)

    def test_finalize_schema_show_inextisting_field(self):
        schema = ATContentTypeSchema.copy()
        # I'm not realy sure what to test
        self.assertNotIn('dummy', schema)
        finalize(schema, show=['dummy'])
        self.assertNotIn('dummy', schema)



from archetypes.schemaextender.field import ExtensionField
from archetypes.schemaextender.interfaces import IOrderableSchemaExtender
from ftw.contentpage import _
from ftw.contentpage.interfaces import IAuthority
from ftw.contentpage.interfaces import IAuthoritySupport
from ftw.contentpage.interfaces import ICategorizable
from Products.Archetypes.public import BooleanField
from Products.Archetypes.public import BooleanWidget
from Products.Archetypes.public import KeywordWidget
from Products.Archetypes.public import LinesField
from zope.component import adapts
from zope.interface import alsoProvides
from zope.interface import implements
from zope.interface import noLongerProvides


class CategoriesField(ExtensionField, LinesField):
    """A reference field to store object topic"""


class CategoriesExtender(object):
    adapts(ICategorizable)
    implements(IOrderableSchemaExtender)

    fields = [CategoriesField(
        'content_categories',
        multiValued=True,
        searchable=False,
        accessor='getContentCategories',
        roleBasedAdd=True,
        widget=KeywordWidget(
            label=_(u'label_categories', default=u'Categories'),
            description=_(u'help_categories',
                          default=u'Category for contentlisting'),
        ),
    ), ]

    def __init__(self, context):
        self.context = context

    def getFields(self):
        return self.fields

    def getOrder(self, original):
        return original


class ListingMarkerCheckbox(ExtensionField, BooleanField):
    """ Listing marker checkbox field """

    def set(self, instance, value, **kwargs):
        """ Copied code from BooleanField, extended by set/unset the marker
        interface.
        """
        if not value or value == '0' or value == 'False':
            value = False
            noLongerProvides(instance, IAuthority)
        else:
            alsoProvides(instance, IAuthority)
            value = True

        self.getStorage(instance).set(self.getName(),
                instance, value, **kwargs)


class ListingMarkerExtender(object):
    adapts(IAuthoritySupport)
    implements(IOrderableSchemaExtender)

    fields = [ListingMarkerCheckbox(
        'mark_as_authority',
        searchable=False,
        required=False,
        default=False,
        write_permission='ftw.contentpage: Toggle IAuthority marker interface',
        widget=BooleanWidget(
            label=_(u'label_mark_as_authority',
                    default=u'Mark content for listings'),
            description=_(u'help_mark_as_authority',
                          default=u'')))]

    def __init__(self, context):
        self.context = context

    def getFields(self):
        return self.fields

    def getOrder(self, original):
        return original



from ftw.upgrade import UpgradeStep


class RemoveIcons(UpgradeStep):
    """Installs the profile which removes the icon expressions from
    simplelayout actions.
    """
    def __call__(self):
        self.setup_install_profile(
            'profile-ftw.contentpage.upgrades:1002')



from AccessControl import ClassSecurityInfo
from Acquisition import aq_inner
from Acquisition import aq_parent
from ftw.contentpage.config import PROJECTNAME
from ftw.contentpage.content.textblock import image_schema
from ftw.contentpage.interfaces import IAuthoritySupport
from ftw.contentpage.interfaces import ICategorizable
from ftw.contentpage.interfaces import IContentPage
from ftw.contentpage.interfaces import ITeaser
from Products.ATContentTypes.content import folder
from Products.ATContentTypes.content import schemata
from Products.CMFCore.permissions import View
from simplelayout.base.interfaces import IAdditionalListingEnabled
from simplelayout.base.interfaces import ISimpleLayoutCapable
from zope.interface import implements

from Products.ATContentTypes.config import HAS_LINGUA_PLONE
if HAS_LINGUA_PLONE:
    from Products.LinguaPlone import public as atapi
else:
    from Products.Archetypes import atapi


ContentPageSchema = folder.ATFolderSchema.copy() + image_schema.copy()

schemata.finalizeATCTSchema(
    ContentPageSchema,
    moveDiscussion=False,
)
ContentPageSchema['relatedItems'].widget.visible['edit'] = 'visible'

# Protect the teaser image with a specific permission
permission = "ftw.contentpage: Edit teaser image on ContentPage"
for name in image_schema.keys():
    ContentPageSchema[name].write_permission = permission


class ContentPage(folder.ATFolder):
    """A simplelayout content page"""
    implements(IContentPage, ICategorizable, ISimpleLayoutCapable,
               IAdditionalListingEnabled, ITeaser, IAuthoritySupport)

    meta_type = "ContentPage"
    schema = ContentPageSchema
    security = ClassSecurityInfo()

    security.declarePublic('canSetDefaultPage')
    def canSetDefaultPage(self):
        return False

    security.declareProtected(View, 'getAvailableLayouts')
    def getAvailableLayouts(self):
        result = super(ContentPage, self).getAvailableLayouts()
        parent = aq_parent(aq_inner(self))
        mid = 'authorities_view'
        # authorities_view is only possible on certain points
        # For example if the ContentPage is created on plone-root.
        # This changes the Plone default behaveiour off setting the layout
        # property on child objects. We don't want this, so let's check for
        # the parent meta_type
        if parent.meta_type == self.meta_type:
            result.remove((mid, mid))
        return result

    security.declarePublic('showAddMenu')
    def showAddMenu(self):
        return False

    security.declarePublic('show_description')
    def show_description(self):
        return False

    security.declarePrivate('copyLayoutFromParent')
    def copyLayoutFromParent(self):
        """Copies the layout from the parent object if it's of the same type.
        """
        # JUST DON'T DO THIS!
        pass


atapi.registerType(ContentPage, PROJECTNAME)



from ftw.builder import Builder
from ftw.builder import create
from ftw.calendar.browser.interfaces import IFtwCalendarEventCreator
from ftw.contentpage.tests import FunctionalTestCase
from ftw.testbrowser import browsing
from zope.component import getMultiAdapter
from DateTime import DateTime


class TestEventCreatorAdapter(FunctionalTestCase):

    def setUp(self):
        super(TestEventCreatorAdapter, self).setUp()
        self.grant('Manager')

    def test_event_creator_adapter_type(self):

        eventCreator = getMultiAdapter((self.portal, self.request),
                                       IFtwCalendarEventCreator)

        self.assertEqual("EventPage", eventCreator.getEventType())

    @browsing
    def test_event_creator_adapter(self, browser):
        event_folder = create(Builder('event folder'))

        eventCreator = getMultiAdapter((event_folder, self.request),
                                       IFtwCalendarEventCreator)

        now = DateTime()

        event = eventCreator.createEvent("test", now)

        self.assertEqual('test', event.Title())
        self.assertEqual(now, event.start())





from ftw.contentpage.config import PROJECTNAME
from Products.CMFCore.utils import getToolByName


def uninstall(self):
    setup_tool = getToolByName(self, 'portal_setup')
    setup_tool.runAllImportStepsFromProfile(
        'profile-{0}:uninstall'.format(PROJECTNAME),
        ignore_dependencies=True)



from ftw.builder import Builder
from ftw.builder import create
from ftw.colorbox.interfaces import IColorboxSettings
from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from plone.registry.interfaces import IRegistry
from unittest2 import TestCase
from zope.component import getUtility
import transaction


class TestListingBlockCreation(TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def setUp(self):
        super(TestListingBlockCreation, self).setUp()
        self.portal = self.layer['portal']
        self.portal_url = self.portal.portal_url()

        registry = getUtility(IRegistry)
        self.colorbox_settings = registry.forInterface(IColorboxSettings)

        self.page = create(Builder('content page'))
        self.listingblock = create(Builder('listing block').within(self.page))
        self.image = create(Builder('image')
                            .with_dummy_content()
                            .within(self.listingblock))

    @browsing
    def test_use_scale_for_large_image(self, browser):
        self.colorbox_settings.image_size = u'colorbox'
        transaction.commit()

        browser.login().visit(self.listingblock, view='block_view-gallery')
        link = browser.css('.sl-img-wrapper a').first

        self.assertEquals(self.get_scaled_img_url(), link.attrib['href'])

    @browsing
    def test_not_scale_defined_returns_origin_image(self, browser):
        self.colorbox_settings.image_size = u''
        transaction.commit()

        browser.login().visit(self.listingblock, view='block_view-gallery')
        link = browser.css('.sl-img-wrapper a').first

        self.assertEquals(self.image.absolute_url(), link.attrib['href'])

    @browsing
    def test_broken_scale_returns_origin_image(self, browser):
        self.colorbox_settings.image_size = u'inexisting'
        transaction.commit()

        browser.login().visit(self.listingblock, view='block_view-gallery')
        link = browser.css('.sl-img-wrapper a').first

        self.assertEquals(self.image.absolute_url(), link.attrib['href'])

    def get_scaled_img_url(self):
        scales = self.image.restrictedTraverse('@@images')
        scaled = scales.scale('image', scale=self.colorbox_settings.image_size)
        return scaled.url



from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from simplelayout.base.interfaces import ISimplelayoutView
from simplelayout.base.viewlets import SimpleLayoutListingViewlet


class SimpleLayoutNewsListingViewlet(SimpleLayoutListingViewlet):

    render = ViewPageTemplateFile('teaser_listing_viewlet.pt')

    def available(self):
        if not ISimplelayoutView.providedBy(self.view):
            return False

        if self.context.Description() or self.context.getImage():
            return True

        return False



from ftw.upgrade import UpgradeStep
from ftw.contentpage.interfaces import IEventPage


class ReindexEventPage(UpgradeStep):

    def __call__(self):
        query = {'object_provides': IEventPage.__identifier__}
        self.catalog_reindex_objects(
            query,
            idxs=['start', 'end']
        )



from DateTime import DateTime
from ftw.builder import Builder
from ftw.builder import create
from ftw.contentpage.tests import FunctionalTestCase
from ftw.testbrowser import browsing


class TestEventArchivePortlet(FunctionalTestCase):

    def setUp(self):
        super(TestEventArchivePortlet, self).setUp()
        self.grant('Manager')

    @browsing
    def test_event_archive_portlet(self, browser):
        event_folder = create(Builder('event folder'))

        create(Builder('event page')
               .within(event_folder)
               .having(startDate=DateTime(2013, 3, 20, 10, 0))
               .having(endDate=DateTime(2013, 3, 20, 11, 0)))

        create(Builder('event page')
               .within(event_folder)
               .having(startDate=DateTime(2013, 5, 26, 10, 0))
               .having(endDate=DateTime(2013, 5, 26, 11, 0)))

        create(Builder('event page')
               .within(event_folder)
               .having(startDate=DateTime(2013, 6, 28, 10, 0))
               .having(endDate=DateTime(2013, 6, 28, 11, 0)))

        create(Builder('event archive portlet').within(event_folder))

        browser.login().visit(event_folder)

        self.assertTrue(
            browser.css('.portlet.portletArchiveListing'),
            msg='There should be an event archive portlet, but it isn\'t.'
        )

        self.assertEqual(
            3,
            len(browser.css('.portlet.portletArchiveListing .portletItem li.month')),
            msg='There should be 3 entries in the event archive portlet.'
        )

    @browsing
    def test_event_archive_portlet_renders_default_title(self, browser):
        event_folder = create(Builder('event folder'))

        create(Builder('event page')
               .within(event_folder)
               .having(startDate=DateTime(2014, 2, 2))
               .having(endDate=DateTime(2014, 2, 6)))

        create(Builder('event archive portlet').within(event_folder))

        browser.login().visit(event_folder)
        self.assertEqual(
            ['Events'],
            browser.css('dl.portletArchiveListing dt').text
        )

    @browsing
    def test_event_archive_portlet_renders_custom_title(self, browser):
        event_folder = create(Builder('event folder'))

        create(Builder('event page')
               .within(event_folder)
               .having(startDate=DateTime(2014, 2, 2))
               .having(endDate=DateTime(2014, 2, 6)))

        create(Builder('event archive portlet').within(event_folder))
        browser.login()

        # Make sure the portlet renders the default title.
        browser.visit(event_folder)
        self.assertEqual(
            ['Events'],
            browser.css('dl.portletArchiveListing dt').text
        )

        # Now let's change the title of the portlet.
        browser.visit(event_folder, view='@@manage-portlets')
        browser.find('Event Archive Portlet').click()
        browser.fill({'Title': u'Foobar'}).submit()

        # Make sure the portlet renders the custom title.
        browser.visit(event_folder)
        self.assertEqual(
            ['Foobar'],
            browser.css('dl.portletArchiveListing dt').text
        )



from plone.app.layout.viewlets import ViewletBase
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class NewsDate(ViewletBase):

    index = ViewPageTemplateFile("news_date.pt")



from ftw.calendar.browser.interfaces import IFtwCalendarEventCreator
from plone import api
from zope.interface import implements


class CalendarEventPageCreator(object):
    implements(IFtwCalendarEventCreator)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def getEventType(self):
        return "EventPage"

    def createEvent(self, title, start_date):
        event = api.content.create(
            container=self.context,
            type=self.getEventType(),
            title=title,
        )

        # Setting the dates does not work via "api.content.create()", reason unknown.
        event.setStartDate(start_date)
        event.setEndDate(start_date)
        event.reindexObject(idxs=['start', 'end'])

        return event



import os
from setuptools import setup, find_packages


version = '1.16.1'
mainainter = 'Mathias Leimgruber'

tests_require = ['ftw.testing [splinter] <1.12.0',
                 'ftw.testbrowser',
                 'ftw.builder',
                 'plone.app.testing',
                 'plone.mocktestcase',
                 'plone.formwidget.contenttree',
                 'ftw.calendar >= 3.0.0',
                 'pyquery',
                 ]

setup(name='ftw.contentpage',
      version=version,
      description="Contentpage based on Simplelayout for for web/intranet",
      long_description=open("README.rst").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),

      # Get more strings from
      # http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.2',
        'Framework :: Plone :: 4.3',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ],

      keywords='ftw contentpage',
      author='4teamwork AG',
      author_email='mailto:info@4teamwork.ch',
      url='https://github.com/4teamwork/ftw.contentpage',
      license='GPL2',

      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['ftw', ],
      include_package_data=True,
      zip_safe=False,
      tests_require=tests_require,
      extras_require=dict(tests=tests_require),

      install_requires=[
          'Plone',
          'archetypes.schemaextender',
          'collective.quickupload',
          'collective.geo.openlayers >= 3.0, < 4.0',
          'collective.geo.mapwidget >= 2.1, < 3.0',
          'ftw.calendarwidget',
          'ftw.colorbox',
          'ftw.geo',
          'ftw.profilehook',
          'ftw.table',
          'ftw.upgrade',
          'plone.api >= 1.3.2',
          'plone.formwidget.contenttree',
          'plone.formwidget.recaptcha',
          'setuptools',
          'simplelayout.base>=4.0.2',
          'simplelayout.ui.base >= 3.0.3',
          'simplelayout.portlet.dropzone',
          'plone.dexterity',
          'plone.directives.form',
          ],

      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



from ftw.contentpage.testing import FTW_CONTENTPAGE_FUNCTIONAL_TESTING
from plone.app.testing import TEST_USER_NAME, TEST_USER_PASSWORD
from plone.testing.z2 import Browser
from simplelayout.base.interfaces import IAdditionalListingEnabled
from simplelayout.base.interfaces import ISimpleLayoutCapable
from unittest2 import TestCase
import transaction
from zope.component import queryMultiAdapter
from zope.publisher.browser import BrowserView
from zope.viewlet.interfaces import IViewletManager


class TestContentPageCreation(TestCase):

    layer = FTW_CONTENTPAGE_FUNCTIONAL_TESTING

    def setUp(self):
        super(TestContentPageCreation, self).setUp()
        self.portal = self.layer['portal']
        self.portal_url = self.portal.portal_url()

        # Browser setup
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False

    def _auth(self):
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_NAME, TEST_USER_PASSWORD, ))

    def test_fti(self):
        self.assertIn('ContentPage', self.portal.portal_types.objectIds())

    def test_creation(self):
        _id = self.portal.invokeFactory('ContentPage', 'contentpage')
        self.assertIn(_id, self.portal.objectIds())

        self._auth()
        self.browser.open('%s/createObject?type_name=ContentPage' %
                          self.portal_url)
        self.browser.getControl("Title").value = 'New ContentPage'
        self.browser.getControl("Save").click()
        self.assertIn("New ContentPage", self.browser.contents)

    def test_getavailablelayouts(self):
        contentpage = self.portal.get(
            self.portal.invokeFactory('ContentPage', 'contentpage'))
        mid = 'authorities_view'
        self.assertIn((mid, mid), contentpage.getAvailableLayouts())

        subpage = contentpage.get(
            contentpage.invokeFactory('ContentPage', 'subpage'))

        # authorities_view should no be available, check comment on
        # ftw.contentpage.content.contentpage -> getAvailableLayouts
        self.assertNotIn((mid, mid), subpage.getAvailableLayouts())

    def test_simplelayout_integration(self):
        contentpage = self.portal.get(
            self.portal.invokeFactory('ContentPage', 'contentpage'))
        ISimpleLayoutCapable.providedBy(contentpage)
        IAdditionalListingEnabled.providedBy(contentpage)

    def test_simplelayout_view(self):
        _id = self.portal.invokeFactory('ContentPage', 'contentpage')
        transaction.commit()
        self._auth()
        self.browser.open(self.portal_url + '/' + _id)
        self.assertIn('template-simplelayout', self.browser.contents)
        self.assertIn('simplelayout-content', self.browser.contents)

    def test_openlayers_is_always_available_on_IContentPage(self):
        contentpage = self.portal.get(
            self.portal.invokeFactory('ContentPage', 'contentpage'))

        view = BrowserView(contentpage, contentpage.REQUEST)
        manager = queryMultiAdapter(
            (contentpage, contentpage.REQUEST, view),
            IViewletManager,
            'plone.htmlhead.links')

        manager.update()

        self.assertIn(
            u'ftw.contentpage.openlayers',
            [v.__name__ for v in manager.viewlets])

    def test_load_openlayer_resources_if_addressblock_is_available(self):
        cp = self.portal.get(
            self.portal.invokeFactory('ContentPage', 'contentpage'))

        view = BrowserView(cp, cp.REQUEST)
        manager = queryMultiAdapter(
            (cp, cp.REQUEST, view),
            IViewletManager,
            'plone.htmlhead.links')
        manager.update()

        openlayer_viewlet = None
        for viewlet in manager.viewlets:
            if viewlet.__name__ == u'ftw.contentpage.openlayers':
                openlayer_viewlet = viewlet

        resource = 'http://maps.google.com/maps/api'
        self.assertNotIn(resource,
                         openlayer_viewlet.render(),
                         'Expect NO google api resource')
        addrblock = cp.get(cp.invokeFactory('AddressBlock', 'addressblock'))
        addrblock.reindexObject()

        self.assertIn(resource,
                      openlayer_viewlet.render(),
                      'Expect google api resource')

    def tearDown(self):
        super(TestContentPageCreation, self).tearDown()
        portal = self.layer['portal']
        if 'contentpage' in portal.objectIds():
            portal.manage_delObjects(['contentpage'])



import logging

logger = logging.getLogger('PloneFlashUpload')
logger.level = logging.getLogger().level

from Acquisition import aq_inner, aq_parent, aq_base
from collective.quickupload.interfaces import IQuickUploadCapable
from collective.quickupload.interfaces import IQuickUploadNotCapable
from Products.Archetypes.utils import shasattr
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.FactoryTool import TempFolder
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class FlashUploadForm(BrowserView):
    """IE drives me cracy. We have to load the form contents with ajax, because
    of this --> https://dev.plone.org/plone/ticket/10894
    Otherwise it will no work with IE < 9.
    IMPORTANT: If this view will not work in the future, consider the JAVASCRIPT
    variable of the quickuploadportlet.py"""

    template = ViewPageTemplateFile('flashupload.pt')

    def __call__(self):
        return self.template()


    def get_upload_url(self):
        """
        return upload url
        in current folder
        """
        ploneview = self.context.restrictedTraverse('@@plone')
        folder_url = ploneview.getCurrentFolderUrl()
        return '%s/@@quick_upload' %folder_url


    def javascript(self):
        return """
  if (jQuery.browser.msie) jQuery("#settings").remove();
  var Browser = {};
  Browser.onUploadComplete = function() {
      window.location.reload();
  }
  loadUploader = function() {
      var ulContainer = jQuery('.quickupload.uploaderContainer');
      ulContainer.each(function(){
          var uploadUrl =  jQuery('.uploadUrl', this).val();
          var uploadData =  'auto';
          var UlDiv = jQuery(this);
          jQuery.ajax({
                     type: 'GET',
                     url: uploadUrl,
                     data: uploadData,
                     dataType: 'html',
                     contentType: 'text/html; charset=utf-8',
                     success: function(html) {
                        if (html.indexOf('quick-uploader') != -1) {
                            UlDiv.html(html);
                        }
                     } });
      });
  }
  jQuery(document).ready(loadUploader);
"""



def isTemporary(obj):
    """Check to see if an object is temporary"""
    if not shasattr(obj, 'isTemporary'):
        return False
    if obj.isTemporary():
        return False

    parent = aq_base(aq_parent(aq_inner(obj)))
    return hasattr(parent, 'meta_type') \
        and parent.meta_type == TempFolder.meta_type


class DisplayUploadView(BrowserView):

    def can_upload(self):

        context = aq_inner(self.context)
        mtool = getToolByName(self.context, 'portal_membership')

        if not IQuickUploadCapable.providedBy(context):
            return False
        elif IQuickUploadNotCapable.providedBy(context):
            return False
        elif not mtool.checkPermission('Add portal content', context):
            return False
        elif isTemporary(context):
            return False

        upload_portal_type = 'auto'
        if (upload_portal_type and upload_portal_type != 'auto'
                and upload_portal_type not in [t.id for t
                        in self.context.getAllowedTypes()]):
            return False
        else:
            return True



from AccessControl import ClassSecurityInfo
from DateTime import DateTime
from ftw.contentpage.config import PROJECTNAME
from ftw.contentpage.content import contentpage
from ftw.contentpage.content.textblock import image_schema
from ftw.contentpage.interfaces import INews
from Products.ATContentTypes.config import HAS_LINGUA_PLONE
from zope.interface import implements


if HAS_LINGUA_PLONE:
    from Products.LinguaPlone.public import registerType
else:
    from Products.Archetypes.atapi import registerType


news_schema = contentpage.ContentPageSchema.copy()

news_schema['effectiveDate'].required = True
news_schema['effectiveDate'].default_method = 'getDefaultEffectiveDate'
news_schema.changeSchemataForField('effectiveDate', 'default')
news_schema.changeSchemataForField('expirationDate', 'default')
news_schema.moveField('image', after='description')


# Protect the teaser image with a specific permission
permission = "ftw.contentpage: Edit teaser image on News"
for name in image_schema.keys():
    news_schema[name].write_permission = permission


class News(contentpage.ContentPage):

    implements(INews)
    security = ClassSecurityInfo()

    schema = news_schema

    def getDefaultEffectiveDate(self):
        return DateTime()


registerType(News, PROJECTNAME)
